#![no_std]
#![no_main]
#![feature(abi_x86_interrupt)]
#![feature(associated_type_defaults)]
#![allow(internal_features)]
#![feature(ptr_internals)]
#![feature(allocator_api)]

use mem::paging::ActivePageTable;
use multiboot2::{BootInformation, BootInformationHeader};
use pc_keyboard::KeyCode;


extern crate alloc;

#[macro_use]
mod logger;
mod interrupts;
mod gdt;
mod utils;
#[macro_use]
mod events;
mod mem;
mod scheduler;
mod std;

fn init(mb2p: *const BootInformationHeader) -> ActivePageTable {
    logger::init(log::LevelFilter::Trace).unwrap();
    gdt::init();
    interrupts::init_idt();
    utils::enable_nxe_bit();
    utils::enable_write_protect_bit();
    std::init();
    mem::init(mb2p)
}

#[no_mangle]
extern "C" fn kmain(mb2p: *const BootInformationHeader) -> ! {
    let _ = init(mb2p); // active table

    let boot_info = unsafe { BootInformation::load(mb2p).unwrap() };
    let fb = boot_info.framebuffer_tag().unwrap().unwrap();
    log::debug!("{fb:?}");
    log::debug!("{}", std::vga::video::is_video_mode_enabled());
    log::info!("end of work");
    loop {
        if std::input::is_key_down(KeyCode::A) {
            log::info!(":3");
        }
    }
    // utils::hcf();
}


#[panic_handler]
fn panic(pi: &core::panic::PanicInfo) -> ! {
    log::error!("{pi}");
    utils::hcf();
}


