mod util;
use core::{borrow::BorrowMut, ops::Deref};

use alloc::boxed::Box;
use lazy_static::lazy_static;
use multiboot2::{BootInformation, BootInformationHeader, FramebufferTag, FramebufferType, TagTrait};
use nalgebra::Vector2;
use spin::{Mutex, MutexGuard};
pub use util::is_video_mode_enabled;



pub struct VideoControler{
    component_size: u8,
    r_ofs: u8,
    g_ofs: u8,
    b_ofs: u8,
    buf_addr: u64,
    size: Vector2<usize>,
}

impl VideoControler {
    pub fn new(mb2p: *const BootInformationHeader) -> Self {
        let boot_info = unsafe { BootInformation::load(mb2p).unwrap() };
        let fbt = boot_info.framebuffer_tag().unwrap().unwrap();
        let FramebufferType::RGB { red, green, blue } = fbt.buffer_type().unwrap() else {
            panic!();
        };
        assert!(red.size == green.size && green.size == blue.size);
        
        Self {
            component_size: red.size,
            r_ofs: red.position,
            g_ofs: green.position,
            b_ofs: blue.position,
            buf_addr: fbt.address(),
            size: Vector2::new(fbt.width() as usize, fbt.height() as usize)
        }
    }
    
}
